@@ -1,12 +1,14 @@
 import pprint as pp
 import time
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Optional
 
 import web3
 from eth_typing.evm import ChecksumAddress
 from tqdm import tqdm
 from web3 import Web3
 
+from moonworm.crawler.ethereum_state_provider import EthereumStateProvider
+
 from .contracts import CU, ERC721
 from .crawler.function_call_crawler import (
     ContractFunctionCall,
@@ -40,12 +42,15 @@ class MockState(FunctionCallCrawlerState):
         self.state = []
 
 
+# TODO(yhtiyar), use state_provider.get_last_block
 def watch_contract(
     web3: Web3,
+    state_provider: EthereumStateProvider,
     contract_address: ChecksumAddress,
     contract_abi: List[Dict[str, Any]],
     num_confirmations: int = 10,
     sleep_time: float = 1,
+    start_block: Optional[int] = None,
 ) -> None:
     """
     Watches a contract for events and calls.
@@ -53,26 +58,30 @@ def watch_contract(
     state = MockState()
     crawler = FunctionCallCrawler(
         state,
-        Web3StateProvider(web3),
+        state_provider,
         contract_abi,
         [web3.toChecksumAddress(contract_address)],
     )
 
     event_abis = [item for item in contract_abi if item["type"] == "event"]
 
-    current_block = web3.eth.blockNumber - num_confirmations * 2
+    if start_block is None:
+        current_block = web3.eth.blockNumber - num_confirmations * 2
+    else:
+        current_block = start_block
+
     progress_bar = tqdm(unit=" blocks")
     progress_bar.set_description(f"Current block {current_block}")
     while True:
         time.sleep(sleep_time)
-        end_block = web3.eth.blockNumber - num_confirmations
+        end_block = min(web3.eth.blockNumber - num_confirmations, current_block + 100)
         if end_block < current_block:
             sleep_time *= 2
             continue
 
         sleep_time /= 2
 
-        crawler.crawl(current_block, end_block)
+        # crawler.crawl(current_block, end_block)
         if state.state:
             print("Got transaction calls:")
             for call in state.state:
